import { defineStore } from "pinia";
import { useConnectionStore } from "./connection";
import * as Y from "yjs";
import { Awareness } from "y-protocols/awareness.js";
import { computed, ref } from "vue";

export const useMissionDetailStore = defineStore("missionDetail", () => {
  const connectionStore = useConnectionStore();

  const ydoc = ref(new Y.Doc());
  const awareness = ref(new Awareness(ydoc.value));

  const editor = ref(ydoc.value.getText("editor"));

  const title = computed({
    get() {
      return ydoc.value.getMap("form").get("title") ?? "";
    },
    set(val) {
      ydoc.value.getMap("form").set("title", val);
    },
  });

  function init(missionId: string) {
    connectionStore.connection?.on("package-sync", (update) => {
      Y.applyUpdate(ydoc.value, new Uint8Array(update));
    });

    connectionStore.connection?.on("package-mission", (initial) => {
      Y.applyUpdate(ydoc.value, new Uint8Array(initial));
    });

    connectionStore.connection?.emit("mission:join", missionId);
  }
  function destroy() {
    connectionStore?.connection?.emit("mission:leave");

    ydoc.value = new Y.Doc();
    awareness.value = new Awareness(ydoc.value);
    editor.value = ydoc.value.getText("editor");
  }

  ydoc.value.on("update", (update) => {
    console.log(connectionStore.connection);
    connectionStore.connection?.emit("mission:sync", Array.from(update));
  });

  return { ydoc, awareness, editor, title, init, destroy };
});
